##############################################################################
#
# Copyright (c) 2016 Projekt01 GmbH.
# All Rights Reserved.
#
##############################################################################
"""Settings
$Id: settings.py 5665 2025-09-18 14:09:06Z roger.ineichen $
"""
from __future__ import absolute_import

__docformat__ = "reStructuredText"

import pymongo.common

import p01.env


################################################################################
#
# mongodb
#
# MONGODB_URI = "mongodb://admin:password@127.0.0.1:27017"
#
# NOTE: MONGODB_URI Port: 45017 is used for testing
#       use your own mongodb uri in your own setup for set the corret uri e.g:
#
#       MONGODB_URI = "mongodb://10.0.0.1:27017"
#       or
#       MONGODB_URI = "mongodb://username:password@127.0.0.1:27017"
#
#       or use the environment variable setup if you need a certificate based
#       setup

MONGODB_URI = "mongodb://127.0.0.1:27017"
MONGODB_CONNECT = False
MONGODB_REPLICA_SET = None
MONGODB_TZ_AWARE = True
MONGODB_APP_NAME = None

# timeout options
MONGODB_MIN_POOL_SIZE = pymongo.common.MIN_POOL_SIZE
MONGODB_MAX_POOL_SIZE = pymongo.common.MAX_POOL_SIZE
MONGODB_CONNECT_TIMEOUT = pymongo.common.CONNECT_TIMEOUT * 1000
MONGODB_SERVER_SELECTION_TIMEOUT = pymongo.common.SERVER_SELECTION_TIMEOUT * 1000
MONGODB_SOCKET_TIMEOUT = 20 * 1000
MONGODB_WAIT_QUEUE_TIMEOUT = 20 * 1000
MONGODB_MAX_IDLE_TIME_MS = pymongo.common.MAX_IDLE_TIME_MS
MONGODB_WAIT_QUEUE_MULTIPLE = None
MONGODB_HEARTBEAT_FREQUENCY = pymongo.common.HEARTBEAT_FREQUENCY * 1000

# ssl options
MONGODB_SSL = False
MONGODB_SSL_CERT_REQUIRED = None
MONGODB_SSL_MATCH_HOSTNAME = True
MONGODB_CA_CERT = None
MONGODB_CLIENT_CERT = None
MONGODB_CLIENT_CERT_KEY_FILE = None
MONGODB_PEM_PASSPHRASE = None
MONGODB_REVOCATION_LIST = None

# tls options
MONGODB_TLS = False
MONGODB_TLS_CA_FILE = None
MONGODB_TLS_CERT_KEY_FILE = None
MONGODB_TLS_CERT_KEY_FILE_PASSWORD = None
MONGODB_TLS_CRL_FILE = None
MONGODB_TLS_ALLOW_INVALID_CERT = False
MONGODB_TLS_ALLOW_INVALID_HOSTNAME = False
MONGODB_TLS_CERTIFICATE_SELECTOR = None

# auth options
MONGODB_AUTH_MECHNISM = None
MONGODB_USERNAME = None
MONGODB_PASSWORD = None


# uri, replica set
MONGODB_URI = p01.env.getEnviron('MONGODB_URI', default=MONGODB_URI)
MONGODB_CONNECT = p01.env.getEnviron('MONGODB_CONNECT', rType=bool,
    default=MONGODB_CONNECT)
MONGODB_REPLICA_SET = p01.env.getEnviron('MONGODB_REPLICA_SET',
    default=MONGODB_REPLICA_SET)
MONGODB_TZ_AWARE = p01.env.getEnviron('MONGODB_TZ_AWARE', rType=bool,
    default=MONGODB_TZ_AWARE)
MONGODB_APP_NAME = p01.env.getEnviron('MONGODB_APP_NAME',
    default=MONGODB_APP_NAME)

# timeout options
MONGODB_CONNECT_TIMEOUT = p01.env.getEnviron('MONGODB_CONNECT_TIMEOUT',
    rType=int, default=MONGODB_CONNECT_TIMEOUT)
MONGODB_SERVER_SELECTION_TIMEOUT = p01.env.getEnviron(
    'MONGODB_SERVER_SELECTION_TIMEOUT', rType=int,
    default=MONGODB_SERVER_SELECTION_TIMEOUT)
MONGODB_SOCKET_TIMEOUT = p01.env.getEnviron('MONGODB_SOCKET_TIMEOUT',
    rType=int, default=MONGODB_SOCKET_TIMEOUT)
MONGODB_WAIT_QUEUE_TIMEOUT = p01.env.getEnviron(
    'MONGODB_WAIT_QUEUE_TIMEOUT', rType=int, default=MONGODB_WAIT_QUEUE_TIMEOUT)
MONGODB_MAX_IDLE_TIME_MS = p01.env.getEnviron(
    'MONGODB_MAX_IDLE_TIME_MS', rType=int, default=MONGODB_MAX_IDLE_TIME_MS)
MONGODB_MIN_POOL_SIZE = p01.env.getEnviron(
    'MONGODB_MIN_POOL_SIZE', rType=int, default=MONGODB_MIN_POOL_SIZE)
MONGODB_MAX_POOL_SIZE = p01.env.getEnviron(
    'MONGODB_MAX_POOL_SIZE', rType=int, default=MONGODB_MAX_POOL_SIZE)
MONGODB_HEARTBEAT_FREQUENCY = p01.env.getEnviron(
    'MONGODB_HEARTBEAT_FREQUENCY', rType=int,
    default=MONGODB_HEARTBEAT_FREQUENCY)
MONGODB_WAIT_QUEUE_MULTIPLE = p01.env.getEnviron('MONGODB_WAIT_QUEUE_MULTIPLE',
    rType=int, default=MONGODB_WAIT_QUEUE_MULTIPLE)

# ssl options
MONGODB_SSL = p01.env.getEnviron('MONGODB_SSL', rType=bool,
    default=MONGODB_SSL)
MONGODB_SSL_CERT_REQUIRED = p01.env.getEnviron(
    'MONGODB_SSL_CERT_REQUIRED', rType=bool, default=MONGODB_SSL_CERT_REQUIRED)
MONGODB_SSL_MATCH_HOSTNAME = p01.env.getEnviron(
    'MONGODB_SSL_MATCH_HOSTNAME', rType=bool,
    default=MONGODB_SSL_MATCH_HOSTNAME)
MONGODB_CA_CERT = p01.env.getEnviron('MONGODB_CA_CERT',
    rType='path', default=MONGODB_CA_CERT)
MONGODB_CLIENT_CERT = p01.env.getEnviron('MONGODB_CLIENT_CERT',
    rType='path', default=MONGODB_CLIENT_CERT)
MONGODB_CLIENT_CERT_KEY_FILE= p01.env.getEnviron('MONGODB_CLIENT_CERT_KEY_FILE',
    rType='path', default=MONGODB_CLIENT_CERT_KEY_FILE)
MONGODB_PEM_PASSPHRASE = p01.env.getEnviron('MONGODB_PEM_PASSPHRASE',
    default=MONGODB_PEM_PASSPHRASE)
MONGODB_REVOCATION_LIST = p01.env.getEnviron(
    'MONGODB_REVOCATION_LIST', rType='path',
    default=MONGODB_REVOCATION_LIST)

# tls options
MONGODB_TLS = p01.env.getEnviron('MONGODB_TLS', rType=bool,
    default=MONGODB_TLS)
MONGODB_TLS_CA_FILE = p01.env.getEnviron('MONGODB_TLS_CA_FILE',
    rType='path', default=MONGODB_TLS_CA_FILE)
MONGODB_TLS_CERT_KEY_FILE = p01.env.getEnviron('MONGODB_TLS_CERT_KEY_FILE',
    rType='path', default=MONGODB_TLS_CERT_KEY_FILE)
MONGODB_TLS_CERT_KEY_FILE_PASSWORD = p01.env.getEnviron(
    'MONGODB_TLS_CERT_KEY_FILE_PASSWORD',
    default=MONGODB_TLS_CERT_KEY_FILE_PASSWORD)
MONGODB_TLS_CRL_FILE = p01.env.getEnviron('MONGODB_TLS_CRL_FILE',
    rType='path', default=MONGODB_TLS_CRL_FILE)
MONGODB_TLS_ALLOW_INVALID_CERT = p01.env.getEnviron(
    'MONGODB_TLS_ALLOW_INVALID_CERT', rType=bool,
    default=MONGODB_TLS_ALLOW_INVALID_CERT)
MONGODB_TLS_ALLOW_INVALID_HOSTNAME = p01.env.getEnviron(
    'MONGODB_TLS_ALLOW_INVALID_HOSTNAME', rType=bool,
    default=MONGODB_TLS_ALLOW_INVALID_HOSTNAME)
MONGODB_TLS_CERTIFICATE_SELECTOR = p01.env.getEnviron(
    'MONGODB_TLS_CERTIFICATE_SELECTOR', rType=str,
    default=MONGODB_TLS_CERTIFICATE_SELECTOR)

# auth options
MONGODB_AUTH_MECHNISM = p01.env.getEnviron('MONGODB_AUTH_MECHNISM',
    default=MONGODB_AUTH_MECHNISM)
MONGODB_USERNAME = p01.env.getEnviron('MONGODB_USERNAME',
    default=MONGODB_USERNAME)
MONGODB_PASSWORD = p01.env.getEnviron('MONGODB_PASSWORD',
    default=MONGODB_PASSWORD)


# support p01.cdn extraction marker
P01_CDN_RECIPE_PROCESS = p01.env.getEnviron('P01_CDN_RECIPE_PROCESS',
    default=False)

# support i18n extraction marker
I18N_EXTRACT = p01.env.getEnviron('I18N_EXTRACT', default=False)



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
from __future__ import absolute_import
from builtins import object
from zope.interface import implementer
__docformat__ = "reStructuredText"

import os.path
import unittest

import pymongo

import zope.interface
import zope.schema
import zope.location.interfaces
from zope.securitypolicy.interfaces import IGrantInfo
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.securitypolicy.interfaces import IPrincipalPermissionManager
from zope.securitypolicy.interfaces import IRolePermissionManager

import m01.stub.testing

import m01.mongo
import m01.mongo.schema
from m01.mongo import interfaces
from m01.mongo import base
from m01.mongo import item
from m01.mongo import storage
from m01.mongo import container
from m01.mongo import security
from m01.mongo.fieldproperty import MongoFieldProperty
from m01.mongo.fieldproperty import MongoDateProperty
from m01.mongo.fieldproperty import MongoGeoLocationProperty
from m01.mongo.fieldproperty import MongoGeoPointProperty
from m01.mongo.fieldproperty import MongoObjectProperty


VERSION = '3.6.23'

# mongo db name used for testing
TEST_DB_NAME = 'm01_mongo_testing'
TEST_COLLECTION_NAME = 'test'
TEST_COLLECTION_FULL_NAME = '%s.%s' % (TEST_DB_NAME, TEST_COLLECTION_NAME)


###############################################################################
#
# test helper methods (moved to m01.fake)
#
###############################################################################

import m01.fake.client
from m01.fake import pprint
from m01.fake import RENormalizer
from m01.fake import reNormalizer


###############################################################################
#
# fake MongoDB connection client
#
###############################################################################

# fake client
from m01.fake import FakeMongoClient
# single shared fake client instance
from m01.fake import fakeMongoClient


###############################################################################
#
# TestCase
#
###############################################################################

class TestCase(unittest.TestCase):

    iface = None
    klass = None

    def getTestInterface(self):
        if self.iface is not None:
            return self.iface
        msg = 'Subclasses has to implement getTestInterface()'
        raise NotImplementedError(msg)

    def getTestClass(self):
        if self.klass is not None:
            return self.klass
        raise NotImplementedError('Subclasses has to implement getTestClass()')

    def getTestData(self):
        return {}

    def makeTestObject(self, data=None):
        # provide default positional or keyword arguments
        if data is None:
            data = self.getTestData()
        testclass = self.getTestClass()
        return testclass(data)


# TODO: implement something like this for test integration
#class MongoIntegrationTestMixin(object):
#    """Mongo integraton test mixin class"""
#
#    def getDumpData(self):
#        return {}
#
#    def test_default_values(self):
#        # integration test
#        obj = self.makeTestObject()
#        schema = self.getTestInterface()
#        fields = zope.schema.getFields(schema)
#        for name, field in fields.items():
#            if field.default != getattr(obj, name):
#                msg = "field %s default %s not equal value %s" %(
#                    name, field.default, getattr(obj, name))
#                raise AssertionError(msg)
#
#    def test_data_integration(self):
#        # integration test
#        obj = self.makeTestObject()
#        dumpData = self.getDumpData()
#        dump = obj.dump()
#        self.assert_(dumpData, dump)


###############################################################################
#
# Public Base Tests
#
###############################################################################

from zope.interface.verify import verifyObject
from zope.interface.verify import verifyClass


class MongoItemBaseTest(TestCase):
    """MongoItem base test"""

    def test_verifyClass(self):
        # class test
        self.assert_(verifyClass(self.getTestInterface(), self.getTestClass()))

    def test_verifyObject(self):
        # object test
        self.assert_(verifyObject(self.getTestInterface(),
            self.makeTestObject()))

    def test_providedBy_IMongoItem(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IMongoItem.providedBy(obj), True)

    def test_mongo_id(self):
        obj = self.makeTestObject()
        self.assertNotEqual(obj._id, None)

    def test_name(self):
        obj = self.makeTestObject()
        self.assertNotEqual(obj.__name__, None)


class MongoObjectBaseTest(TestCase):
    """MongoItem base test"""

    def test_verifyClass(self):
        # class test
        self.assert_(verifyClass(self.getTestInterface(), self.getTestClass()))

    def test_verifyObject(self):
        # object test
        self.assert_(verifyObject(self.getTestInterface(),
            self.makeTestObject()))

    def test_providedBy_IMongoObject(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IMongoObject.providedBy(obj), True)

    def test_mongo_id(self):
        obj = self.makeTestObject()
        self.assertNotEqual(obj._id, None)

    def test_name(self):
        obj = self.makeTestObject()
        self.assertNotEqual(obj.__name__, None)


class MongoSubItemBaseTest(TestCase):
    """MongoSubItem base test"""

    def test_verifyClass(self):
        # class test
        self.assert_(verifyClass(self.getTestInterface(), self.getTestClass()))

    def test_verifyObject(self):
        # object test
        self.assert_(verifyObject(self.getTestInterface(),
            self.makeTestObject()))

    def test_providedBy_IMongoSubItem(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IMongoSubItem.providedBy(obj), True)

    def test_mongo_id(self):
        obj = self.makeTestObject()
        self.assertNotEqual(obj._id, None)

    def test_name(self):
        obj = self.makeTestObject()
        self.assertNotEqual(obj.__name__, None)


class MongoContainerBaseTest(TestCase):
    """MongoContainer base test"""

    def test_verifyClass(self):
        # class test
        self.assert_(verifyClass(self.getTestInterface(), self.getTestClass()))

    def test_verifyObject(self):
        # object test
        self.assert_(verifyObject(self.getTestInterface(),
            self.makeTestObject()))

    def test_providedBy_IMongoContainer(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IMongoContainer.providedBy(obj), True)


class MongoStorageBaseTest(TestCase):
    """MongoStorage base test"""

    def test_verifyClass(self):
        # class test
        self.assert_(verifyClass(self.getTestInterface(), self.getTestClass()))

    def test_verifyObject(self):
        # object test
        self.assert_(verifyObject(self.getTestInterface(),
            self.makeTestObject()))

    def test_providedBy_IMongoStorage(self):
        obj = self.makeTestObject()
        self.assert_(interfaces.IMongoStorage.providedBy(obj), True)


###############################################################################
#
# test helper methods
#
###############################################################################

_testClient = None

def getTestClient():
    return _testClient


def getTestDatabase():
    client = getTestClient()
    return client[TEST_DB_NAME]


def getTestCollection():
    database = getTestDatabase()
    return database[TEST_COLLECTION_NAME]


def dropTestDatabase():
    client = getTestClient()
    client.drop_database(TEST_DB_NAME)


def dropTestCollection():
    client = getTestClient()
    client[TEST_DB_NAME].drop_collection(TEST_COLLECTION_NAME)


###############################################################################
#
# test setup methods
#
###############################################################################

# fake mongodb setup
def setUpFakeMongo(test=None):
    """Setup fake (singleton) mongo client"""
    global _testClient
    host = 'localhost'
    port = 45017
    tz_aware = True
    storage = m01.fake.client.DatabaseStorage
    _testClient = FakeMongoClient(host, port, tz_aware=tz_aware,
        storage=storage)


def tearDownFakeMongo(test=None):
    """Tear down fake mongo client"""
    # reset test client
    global _testClient
    _testClient = None
    # clear thread local transaction cache
    m01.mongo.clearThreadLocalCache()


# stub mongodb server
def setUpStubMongo(test=None):
    """Setup pymongo client as test client and setup a real empty mongodb"""
    host = 'localhost'
    port = 45017
    version = VERSION
    tz_aware = True
    sandBoxDir = os.path.join(os.path.dirname(__file__), 'sandbox')
    import m01.stub.testing
    m01.stub.testing.startMongoServer(host, port, sandBoxDir=sandBoxDir,
        version=version)
    # setup pymongo.MongoClient as test client
    global _testClient
    _testClient = pymongo.MongoClient(host, port, tz_aware=tz_aware)


def tearDownStubMongo(test=None):
    """Tear down real mongodb"""
    # stop mongodb server
    sleep = 0.5
    import m01.stub.testing
    m01.stub.testing.stopMongoServer(sleep)
    # reset test client
    global _testClient
    _testClient = None
    # clear thread local transaction cache
    m01.mongo.clearThreadLocalCache()


##############################################################################
#
# test setup helper
#
##############################################################################

# security policy testing setup
def setUpSecurityPolicyAdapters():
    zope.component.provideAdapter(security.PrincipalPermissionManager,
        (interfaces.ISecurityAware,), IPrincipalPermissionManager)
    zope.component.provideAdapter(security.PrincipalRoleManager,
        (interfaces.ISecurityAware,), IPrincipalRoleManager)
    zope.component.provideAdapter(security.RolePermissionManager,
        (interfaces.ISecurityAware,), IRolePermissionManager)
    zope.component.provideAdapter(security.GrantInfoAdapter,
        (interfaces.ISecurityAware,), IGrantInfo)


##############################################################################
#
# test components
#
##############################################################################

class TestCollectionMixin(object):
    """Fake test collection mixin class"""

    @property
    def collection(self):
        db = getTestDatabase()
        return db['test']


# MongoSubItem
class ISampleSubItem(interfaces.IMongoSubItem):
    """Sample sub item interface."""

    text = zope.schema.TextLine(
        title=u'Text',
        description=u'Text',
        default=u'',
        required=True)


@zope.interface.implementer(ISampleSubItem)
class SampleSubItem(item.MongoSubItem):
    """Sample sub item."""


    dumpNames = ['text']

    text = MongoFieldProperty(ISampleSubItem['text'])


# test schema for MongoItem with sub item
class ITestSchema(zope.interface.Interface):
    """Basic sample schema."""

    title = zope.schema.TextLine(
        title=u'Title',
        description=u'Title',
        default=u'',
        required=True)

    description = zope.schema.Text(
        title=u'Description',
        description=u'Description',
        default=u'',
        required=False)

    item = zope.schema.Object(
        title=u'Mongo Item',
        description=u'Mongo Item',
        schema=ISampleSubItem,
        default=None,
        required=False,
        )

    number = zope.schema.Int(
        title=u'Number',
        description=u'Number',
        default=None,
        required=False)

    numbers = m01.mongo.schema.MongoList(
        title=u'Numbers',
        description=u'Numbers',
        value_type=zope.schema.Int(
            title=u'Number',
            description=u'Number',
            ),
        default=[],
        required=False)

    comments = m01.mongo.schema.MongoList(
        title=u'Comments',
        description=u'Comments',
        value_type=zope.schema.Object(
            title=u'Comment',
            description=u'Comment',
            schema=ISampleSubItem,
            default=None,
            required=False,
            ),
        default=[],
        required=False)

    date = m01.mongo.schema.MongoDate(
        title=u'Date',
        description=u'Date',
        required=False)


# MongoStorage
class ISampleStorageItem(ITestSchema, interfaces.IMongoStorageItem):
    """Sample item interface."""

    __name__ = zope.schema.TextLine(
        title=u'Title',
        description=u'Title',
        missing_value=u'',
        default=None,
        required=True)


@zope.interface.implementer(ISampleStorageItem)
class SampleStorageItem(item.MongoStorageItem, base.MongoUpdateMixin):
    """Sample item."""


    title = MongoFieldProperty(ISampleStorageItem['title'])
    description = MongoFieldProperty(ISampleStorageItem['description'])
    item = MongoFieldProperty(ISampleStorageItem['item'])
    number = MongoFieldProperty(ISampleStorageItem['number'])
    numbers = MongoFieldProperty(ISampleStorageItem['numbers'])
    comments = MongoFieldProperty(ISampleStorageItem['comments'])
    date = MongoDateProperty(ISampleStorageItem['date'])

    dumpNames = ['title', 'description', 'item',  'numbers', 'number',
                 'comments', 'date']

    converters = {'item': SampleSubItem,
                  'comments': SampleSubItem}


class ISampleStorage(interfaces.IMongoStorage):
    """Sample storage interface."""


@zope.interface.implementer(ISampleStorage)
class SampleStorage(TestCollectionMixin, storage.MongoStorage):
    """Sample storage."""


    def __init__(self):
        pass

    def load(self, data):
        """Load data into the right mongo item."""
        return SampleStorageItem(data)


# MongoContainer
class ISampleContainerItem(ITestSchema, interfaces.IMongoContainerItem,
    zope.location.interfaces.ILocation):
    """Sample item interface."""

    __name__ = zope.schema.TextLine(
        title=u'Title',
        description=u'Title',
        missing_value=u'',
        default=None,
        required=True)


@zope.interface.implementer(ISampleContainerItem)
class SampleContainerItem(item.MongoContainerItem):
    """Sample item."""


    __name__ = MongoFieldProperty(ISampleContainerItem['__name__'])
    title = MongoFieldProperty(ISampleContainerItem['title'])
    description = MongoFieldProperty(ISampleContainerItem['description'])
    item = MongoFieldProperty(ISampleContainerItem['item'])
    number = MongoFieldProperty(ISampleContainerItem['number'])
    numbers = MongoFieldProperty(ISampleContainerItem['numbers'])
    comments = MongoFieldProperty(ISampleContainerItem['comments'])
    date = MongoDateProperty(ISampleContainerItem['date'])

    dumpNames = ['title', 'description', 'item',  'numbers', 'number',
                 'comments', 'date']

    converters = {'item': SampleSubItem,
                  'comments': SampleSubItem}


class ISampleContainer(interfaces.IMongoContainer):
    """Sample container interface."""


@zope.interface.implementer(ISampleContainer)
class SampleContainer(TestCollectionMixin, container.MongoContainer):
    """Sample container."""


    def load(self, data):
        """Load data into the right mongo item."""
        return SampleContainerItem(data)


# MongoObject
class ISampleMongoObject(ITestSchema, interfaces.IMongoObject):
    """Sample mongo object interface."""


@zope.interface.implementer(ISampleMongoObject)
class SampleMongoObject(item.MongoObject):
    """Sample mongo object."""


    title = MongoFieldProperty(ISampleMongoObject['title'])
    description = MongoFieldProperty(ISampleMongoObject['description'])
    item = MongoFieldProperty(ISampleMongoObject['item'])
    number = MongoFieldProperty(ISampleMongoObject['number'])
    numbers = MongoFieldProperty(ISampleMongoObject['numbers'])
    comments = MongoFieldProperty(ISampleMongoObject['comments'])
    date = MongoDateProperty(ISampleMongoObject['date'])

    dumpNames = ['title', 'description', 'item',  'numbers', 'number',
                 'comments', 'date']

    converters = {'item': SampleSubItem,
                  'comments': SampleSubItem}

    @classmethod
    def getCollection(cls, parent):
        conn = fakeMongoClient
        return conn['m01MongoTesting']['m01MongoObjectTest']


class IContent(zope.interface.Interface):
    """Sample content interface."""

    _moid = zope.schema.Int(
        title=u'Object ID',
        )

    obj = zope.schema.Object(
        title=u'Mongo Object',
        schema=ISampleMongoObject,
        )

class Content(object):
    """Sample content (could be a zope persistent object)."""

    _m_initialized = None

    # could but must not be a zope persistent property
    _moid = zope.schema.fieldproperty.FieldProperty(IContent['_moid'])

    # mongo object property
    obj = MongoObjectProperty(IContent['obj'], SampleMongoObject)

    def __init__(self, _moid):
        self._moid = _moid
        self._m_initialized = True

    def __repr__(self):
        return u'<%s %s>' % (self.__class__.__name__, self._moid)


###############################################################################
#
# complex company sample data
#
###############################################################################

# collections
def getEmployers():
    db = getTestDatabase()
    return db['employers']

def getCompanies():
    db = getTestDatabase()
    return db['companies']

def getRootItems():
    db = getTestDatabase()
    return db['items']

# document
class IDocument(interfaces.IMongoContainerItem):
    """Sample sub item interface."""

    text = zope.schema.TextLine(
        title=u'Text',
        description=u'Text',
        required=True)

@zope.interface.implementer(IDocument)
class Document(item.MongoContainerItem):
    """Sample sub item."""


    dumpNames = ['text']

    text = MongoFieldProperty(IDocument['text'])


# employer
class IEmployer(interfaces.IMongoContainerItem, interfaces.IMongoContainer):
    """Sample container interface."""

    name = zope.schema.TextLine(
        title=u'Name',
        description=u'Name',
        required=True)


class Employer(item.MongoContainerItem, container.MongoContainer):
    """Mongo company item and Employer container"""

    name = MongoFieldProperty(IEmployer['name'])

    dumpNames = ['name']

    @property
    def cacheKey(self):
        return self.__name__

    @property
    def collection(self):
        return getEmployers()

    def load(self, data):
        """Load data into the right mongo item."""
        return Document(data)


# company
class ICompany(interfaces.IMongoContainerItem, interfaces.IMongoContainer):
    """Sample container interface."""

    name = zope.schema.TextLine(
        title=u'Name',
        description=u'Name',
        required=True)

class Company(item.MongoContainerItem, container.MongoContainer):
    """Mongo company item and Employer container"""

    name = MongoFieldProperty(ICompany['name'])

    dumpNames = ['name']

    @property
    def cacheKey(self):
        return self.__name__

    @property
    def collection(self):
        return getEmployers()

    def load(self, data):
        """Load data into the right mongo item."""
        return Employer(data)


# companies
class ICompanies(interfaces.IMongoContainerItem, interfaces.IMongoContainer):
    """Sample container interface."""

    name = zope.schema.TextLine(
        title=u'Name',
        description=u'Name',
        required=True)

class Companies(item.MongoContainerItem, container.MongoContainer):
    """Mongo company container"""

    name = MongoFieldProperty(ICompanies['name'])

    dumpNames = ['name']

    @property
    def cacheKey(self):
        return self.__name__

    @property
    def collection(self):
        return getCompanies()

    def load(self, data):
        """Load data into the right mongo item."""
        return Company(data)


class IGeoSample(interfaces.IMongoContainerItem):
    """Geo sample interface."""

    name = zope.schema.TextLine(
        title=u'Name',
        description=u'Name',
        required=True)

    lonlat = zope.schema.Object(
        title=u'Location',
        description=u'Location',
        schema=interfaces.IGeoLocation,
        required=False)


@zope.interface.implementer(IGeoSample)
class GeoSample(item.MongoContainerItem):
    """Geo sample item"""


    name = MongoFieldProperty(IGeoSample['name'])
    lonlat = MongoGeoLocationProperty(IGeoSample['lonlat'])

    dumpNames = ['name', 'lonlat']


class IGeoPointSample(interfaces.IMongoContainerItem):
    """GeoPoint sample interface."""

    name = zope.schema.TextLine(
        title=u'Name',
        description=u'Name',
        required=True)

    lonlat = zope.schema.Object(
        title=u'GeoPoint',
        description=u'GeoPoint',
        schema=interfaces.IGeoPoint,
        required=False)


@zope.interface.implementer(IGeoPointSample)
class GeoPointSample(item.MongoContainerItem):
    """GeoPoint sample item"""


    name = MongoFieldProperty(IGeoPointSample['name'])
    lonlat = MongoGeoPointProperty(IGeoPointSample['lonlat'])

    dumpNames = ['name', 'lonlat']



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""
$Id:$
"""
from __future__ import absolute_import

__docformat__ = "reStructuredText"

import zope.interface
from zope.container import contained

from m01.mongo import interfaces
from m01.mongo import base


@zope.interface.implementer(interfaces.IMongoContainer)
class MongoContainer(base.MongoContainerBase, contained.Contained):
    """Mongo storage based container supporting item with __name__.
    
    This class can be used without the ZODB.

    Note: don't forget to mixin persistent in your MongoContainer class
    if you like to store such MongoContainer in ZODB.
    """




##############################################################################
#
# Copyright (c) 2009 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
from __future__ import division
from __future__ import absolute_import
from six import string_types
from past.utils import old_div
import six
__docformat__ = "reStructuredText"

import pymongo


def getBatchData(collection, query=None, page=1, size=25, sortName=None,
    sortOrder=None, fields=None):
    """Returns batched mongo cursor, current page, total items and page size.
    
    Note: this method will not return loaded IMongoItems. If you need to
    load the given mongo data you can simply iterate the given cursor and
    call doLoad(data). The doLoad method also takes care of all the cached
    items. But take care, since mongo returns removed items and doesn't
    know about new added items till a transaction commits, this method
    could return bad items. And if you like to skip removed items from
    the curser by yourself, the page and pages counter could get messed up.
    If you like to bypass this troubles, I highly recommend only call
    doLoad based on cursor data if you didn't add or remove items from a
    container in the same transaction.
    (response.redirect or transaction.commit are your friends)

    Note: the page can get recalculated if the page position doesn't fit
    into the batch. This means the page in the reult can be different as
    the given page input. This is usefull if we use a larger page number
    as we realy have, e.g. after remove an item.

    Note: this method will call doBatchDataFilter which could enhance
    the given query specification

    Note: take care, this method will ignore cached (removed, added) items
    till we commit the transaction. You can eighter commit the transaction
    or use a response redirect in your request before using this method
    after remove or add items.
    """
    # start with server, this allows us to approve the page position
    if query is None:
        query = {}
    cursor = collection.find(query, fields)

    # get overall total based on query
    total = cursor.count() 
    # calculate pages
    pages = old_div(total,size)
    if pages == 0 or total % size:
        pages += 1
    # as next we approve our page position
    if page > pages:
        # restart with pages number as page which is the last page
        page = pages
        return getBatchData(collection, query, page, size, sortName, sortOrder,
            fields)

    # calculate start size
    start = (page-1) * size
    cursor = cursor.skip(start)
    # sort result
    if sortName is not None:
        if sortOrder is None:
            if isinstance(sortName, six.string_types):
                sortName = [(sortName, pymongo.ASCENDING)]
            elif not isinstance(sortName, list):
                raise TypeError("if no sortOrder is specified, "
                                "sortName must be an instance of list")
        cursor = cursor.sort(sortName, sortOrder)
    # limit result
    cursor = cursor.limit(size)
    # return data including probably adjusted page number
    return (cursor, page, pages, total)



###############################################################################
#
# Copyright (c) 2011 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
###############################################################################
"""Geo location item
$Id:$
"""
from __future__ import absolute_import

__docformat__ = "reStructuredText"

import bson.son

import zope.interface
import zope.location.location
from zope.security.proxy import removeSecurityProxy

from m01.mongo import interfaces
from m01.mongo.fieldproperty import MongoFieldProperty


class GeoBase(zope.location.location.Location):
    """Geo base class"""

    __parent__ = None
    __name__ = None
    _m_initialized = False
    _m_changed_value = None

    _lon = MongoFieldProperty(interfaces.IGeoSchema['lon'])
    _lat = MongoFieldProperty(interfaces.IGeoSchema['lat'])

    def _m_changed():
        def get(self):
            return self._m_changed_value
        def set(self, value):
            if value == True:
                self._m_changed_value = value
                if self.__parent__ is not None:
                    removeSecurityProxy(self.__parent__)._m_changed = value
            elif value != True:
                raise ValueError("Can only dispatch True to __parent__")
        return property(get, set)

    _m_changed = _m_changed()

    def lat():
        def fget(self):
            return self._lat
        def fset(self, value):
            if self._lat != value:
                # prevent write access if not changed
                self._lat = value
        return property(fget, fset)

    lat = lat()

    def lon():
        def fget(self):
            return self._lon
        def fset(self, value):
            if self._lon != value:
                # prevent write access if not changed
                self._lon = value
        return property(fget, fset)

    lon = lon()

    def __repr__(self):
        return u'<%s lon:%s, lat:%s>' % (self.__class__.__name__, self.lon,
            self.lat)


# IGeoLocation
@zope.interface.implementer(interfaces.IGeoLocation)
class GeoLocation(GeoBase):
    """Geo location data wrapper, providing preserved lon/lat value order
    and supporting _m_changed marker concept.

    This item must get defined with a zope.schema.Object field and the
    MongoGeoLocationProperty property. for more info see geo.txt and testing.py

    IMPORTANT:
    we use the order latitude, longitude and (NOT lon, lat) like described in
    mongodb documentation

    A more detailed description about the lon/lat order thema:

    looking at the Geographic Coordinate System, you'll notice that longitude
    counts left to right, while latitude counts bottom to top. Translating that
    into x/y this would mean the Longitude/Latitude approach is the only one
    that makes sense logically.

    In english, coordinates are usually referred to as latitude/longitude in a
    term. This is due to 2 reasons: a precedents o in the alphabet and in
    Navigation, in the days of sextants Latitude was easier, and more exactly
    determined, so that was the first thing to determine/write down.

    From OGC Specs, lon/lat is the correct way to interpret them. You can
    easily see that from how those that where participants in creating the
    Specs (Oracle/Informix/ESRI/IBM) implemented them, and those that did
    their own stuff (MS/Google).

    """


    asDict = True

    def __init__(self, data):
        """Initialize a MongoSubItem with given data."""
        if (isinstance(data, list) and len(data) == 2):
            self.asDict = False
            # set lon, lat
            lon = data[0]
            lat = data[1]
        else:
            self.asDict = True
            # set given or None __parent__, we will set the __parent__ later
            # within the field property if not given yet.
            self.__parent__ = data.pop('__parent__', None)
            # set lon, lat
            lon = data.get('lon', None)
            lat = data.get('lat', None)

        # set lon, lat values and ensure float values
        if lon is not None:
            self.lon = float(lon)
        if lat is not None:
            self.lat = float(lat)
        # mark as not changed
        self._m_changed_value = None
        # mark the item as initialized. This allows us to implement attribtues
        # with apply decorator which act different for an initialized object
        self._m_initialized = True

    def dump(self, dumpNames=None, dictify=False):
        """Dump to data dict prevent lon/lat order by using SON """
        if self.asDict or dictify:
            # bson.son.SON will ensure the order in the dict
            data = bson.son.SON()
            if self.lon is not None:
                data['lon'] = self.lon
            if self.lat is not None:
                data['lat'] = self.lat
            return data
        else:
            return [self.lon, self.lat]


@zope.interface.implementer(interfaces.IGeoPoint)
class GeoPoint(GeoBase):
    """Geo Point data wrapper, providing preserved lon/lat value order
    and supporting _m_changed marker concept.

    This item must get defined with a zope.schema.Object field and the
    MongoGeoPointProperty property. For more info see geo.txt and testing.py

    This implementation uses the order longitude, latitude as described in
    mongodb documentation because of unnamed lng/lat list values.

    """


    def __init__(self, data):
        """Initialize a MongoSubItem with given data."""
        lon = None
        lat = None
        if isinstance(data, (dict, bson.son.SON)):
            if data.get('coordinates'):
                # GeoJSON format from mongodb
                type = data.get('type')
                if type is not None and type != self.type:
                    raise TypeError("Wrong GoePoint type given", type)
                # set given or None __parent__, we will set the __parent__ later
                # within the field property if not given yet.
                self.__parent__ = data.pop('__parent__', None)
                # set coordinates if given
                coordinates = data.get('coordinates')
                if coordinates and len(coordinates) == 2:
                    # set lon, lat
                    lon = coordinates[0]
                    lat = coordinates[1]
            else:
                # simple dict with lon, lat given (see MongoGeoPointProperty)
                self.__parent__ = data.pop('__parent__', None)
                # set lon, lat
                lon = data.get('lon', None)
                lat = data.get('lat', None)
        elif (isinstance(data, list) and len(data) == 2):
            # simple lon, lat list given (see MongoGeoPointProperty)
            self.asDict = False
            # set lon, lat
            lon = data[0]
            lat = data[1]

        # set lon, lat values and ensure float values
        if lon is not None and lat is not None:
            self.lon = float(lon)
            self.lat = float(lat)
        # mark as not changed
        self._m_changed_value = None
        # mark the item as initialized. This allows us to implement attribtues
        # with apply decorator which act different for an initialized object
        self._m_initialized = True

    def dump(self, dumpNames=None, dictify=False):
        """Dump to data dict prevent lon/lat order by using SON """
        # bson.son.SON will ensure the order in the dict
        data = bson.son.SON()
        data['type'] = self.type
        if self.lon is not None and self.lat is not None:
            data['coordinates'] = [self.lon, self.lat]
        if dictify:
            data = m01.mongo.dictify(data)
        return data

    @property
    def type(self):
        return u'Point'


##############################################################################
#
# Copyright (c) 2008 Projekt01 GmbH and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""

from __future__ import absolute_import
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup (
    name='m01.mongo',
    version='3.13.6',
    author = "Roger Ineichen, Projekt01 GmbH",
    author_email = "dev@projekt01.ch",
    description = "MongoDB connection pool and container implementation for Zope3",
    long_description=(
        read('README.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'README.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'container.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'storage.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'shared.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'object.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'geo.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'geopoint.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'batching.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'testing.txt')
        + '\n\n' +
        read('src', 'm01', 'mongo', 'tests', 'advanced.txt')
        + '\n\n' +
        read('CHANGES.txt')
        ),
    license = "ZPL 2.1",
    keywords = "Zope3 z3c p01 m01 mongo connection pool container",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope3'],
    url = 'http://pypi.python.org/pypi/m01.mongo',
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['m01'],
    extras_require=dict(
        test = [
            'm01.fake',
            'm01.stub',
            'zope.testing',
        ],
        performance = [
            'ZODB',
            'm01.fake',
            'm01.stub',
            'zope.testing',
        ]),
    install_requires = [
        'setuptools',
        'pymongo >= 2.4.1',
        'p01.env',
        'zope.component',
        'zope.container',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.lifecycleevent',
        'zope.location',
        'zope.schema',
        'zope.security',
        'zope.securitypolicy',
        ],
    zip_safe = False,
    )
